#!/usr/bin/env ruby

require 'net/http'
require 'json'
require 'zlib'

K_BILIBILI_API_SPACE_URL = 'space.bilibili.com'
K_BILIBILI_API_SPACE_MEMBER_QUERY = '/ajax/member/GetInfo?mid='
K_BILIBILI_API_SPACE_FANS_QUERY = '/ajax/friend/GetFansList?mid='
K_BILIBILI_API_SPACE_COINVIDEO_QUERY = '/ajax/member/getCoinVideos?pagesize=100&mid='
K_BILIBILI_API_SPACE_SUBSCRIBE_QUERY = '/ajax/tags/getList?mid='
K_BILIBILI_API_SPACE_SUBSCRIBE_TAGS_QUERY = '/ajax/tags/getSubList?mid='
K_BILIBILI_API_SPACE_TAGS_QUERY = '/ajax/member/getTags?mids='
K_BILIBILI_API_SPACE_SUBMITVIDEOS = '/ajax/member/getSubmitVideos?mids='
K_BILIBILI_API_SPACE_ATTENTION_LIST = '/ajax/friend/GetAttentionList?mid='
K_BILIBILI_API_SPACE_BOX_LIST = '/ajax/fav/getBoxList?mid='

K_BILIBILI_API_MAIN_URL = 'www.bilibili.com'
K_BILIBILI_API_INDEX_BANGUMI = '/index/index-bangumi-timeline.json'

K_BILIBILI_API_URL = 'api.bilibili.com'
K_BILIBILI_API_VIDEO_SHARE = '/x/share/count?actionKey=appkey&appkey=27eb53fc9058f8c3&aid='

K_BILIBILI_COMMENT_HOST = 'http://comment.bilibili.com/'
K_BILIBILI_API_COMMENT_URL = 'comment.bilibili.com'

K_BILIBILI_API_INTERFACE_URL = 'interface.bilibili.com'
K_BILIBILI_API_VIDEO_DOWNLOAD = '/playurl?otype=json&appkey=0a99fa1d87fdd38c&platform=ios&quality='

class Member
    attr_accessor :mid                    # 用户id
    attr_accessor :name                   # 昵称
    attr_accessor :approve                # 是否为认证用户
    attr_accessor :sex                    # 性别
    attr_accessor :rank                   # 排名
    attr_accessor :face                   # 头像URL
    attr_accessor :coins                  # 硬币数
    attr_accessor :displayRank            # 显示排名
    attr_accessor :regtime                # 注册时间
    attr_accessor :spacesta               # 理解不能
    attr_accessor :birthday               # 生日
    attr_accessor :place                  # 地点
    attr_accessor :description            # 描述
    attr_accessor :article                # 理解不能
    attr_accessor :attentions             # TA关注的人
    attr_accessor :fansCount              # 关注TA的人数
    attr_accessor :friend                 # TA关注的人数
    attr_accessor :attention              # TA关注的人数
    attr_accessor :sign                   # 签名
    attr_accessor :current_level          # 当前级别
    attr_accessor :current_min            # 当前级别起始经验
    attr_accessor :current_exp            # TA的经验
    attr_accessor :next_exp               # 到达下一级需要的经验
    
    def Member.member_with_JSON(json)
        data = json['data']
        user = Member.new
        user.mid = data['mid']
        user.name = data['name']
        user.approve = data['approve']
        user.sex = data['sex']
        user.rank = data['rank']
        user.face = data['face']
        user.coins = data['coins']
        user.displayRank = data['displayRank']
        user.regtime = data['regtime']
        user.spacesta = data['spacesta']
        user.birthday = data['birthday']
        user.place = data['place']
        user.description = data['description']
        user.article = data['article']
        user.attentions = data['attentions']
        user.fansCount = data['fans']
        user.friend = data['friend']
        user.attention = data['attention']
        user.sign = data['sign']
        user.current_level = data['level_info']['current_level']
        user.current_min = data['level_info']['current_min']
        user.current_exp = data['level_info']['current_exp']
        user.next_exp = data['level_info']['next_exp']
        user
    end
    
    # 投稿视频
    def submitVideos(tid = 0)
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_SUBMITVIDEOS + @mid + "&tid=" + tid.to_s)
        JSON.parse(response.body)
    end
    
    # 我的收藏
    def collectionList
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_BOX_LIST + @mid)
        JSON.parse(response.body)
    end
    
    # 订阅番剧
    def subscribeList
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_SUBSCRIBE_QUERY + @mid)
        JSON.parse(response.body)
    end
    
    # 订阅标签
    def subscribeTags
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_SUBSCRIBE_TAGS_QUERY + @mid)
        JSON.parse(response.body)
    end
    
    # 投过币的视频
    def coinVideos
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_COINVIDEO_QUERY + @mid)
        JSON.parse(response.body)
    end
    
    # 关注列表
    def attentionList
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_ATTENTION_LIST + @mid)
        JSON.parse(response.body)
    end
    
    # 粉丝
    def fans
        response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_FANS_QUERY + @mid)
        JSON.parse(response.body)
    end
end

class Video
    attr_accessor :title                    # 视频标题
    attr_accessor :time                     # 视频发布时间
    attr_accessor :up                       # up主的用户id
    attr_accessor :videos                   # 分p
    attr_accessor :cid                      # 视频cid
    attr_accessor :aid                      # 视频aid
    attr_accessor :commentURL               # 弹幕URL

    def Video.av(aid = nil)
        response = Net::HTTP.new(K_BILIBILI_API_MAIN_URL).get('/video/av' + aid.to_s + '/')
        
        video = Video.new
        video.videos = Array.new
        video.aid = aid
        
        response.body.each_line { |line| 
            t = /<div class="v-title"><h1 title="(.*?)">/.match(line);
            if t != nil
                video.title = t[1]
            end
            
            t = /<i>(\d{4}-\d{2}-\d{2} \d{2}:\d{2})<\/i><\/time>/.match(line);
            if t != nil
                video.time = t[1]
            end
            
            t = /<div class="usname"><a class="name" href="http:\/\/space.bilibili.com\/(\d+)"/.match(line);
            if t != nil
                video.up = t[1]
            end
            
            t = /<script type='text\/javascript'>EmbedPlayer\('player', "http:\/\/static.hdslb.com\/play.swf", "cid=(\d+)(?:.*?)"/.match(line);
            if t != nil
                video.cid = t[1]
                video.commentURL = K_BILIBILI_COMMENT_HOST + video.cid + '.xml'
            end
            
            t = /<option value='(.*?)'>(.*?)<\/option>/.match(line);
            if t != nil
                sp = Hash.new
                sp['URL'] = 'http://www.bilibili.com' + t[1]
                sp['title'] = t[2]
                video.videos.push(sp)
            end
        }
        return video
    end

    # 视频分享量
    def Video.shareCount(aid)
        response = Net::HTTP.new(K_BILIBILI_API_URL).get(K_BILIBILI_API_VIDEO_SHARE + aid.to_s)
        result = JSON.parse(response.body)
        if result['code'] == 0
            return result['data']
        end
    end
    
    # 通过av号获取cid
    def Video.avToCID(aid)
        response = Net::HTTP.new(K_BILIBILI_API_MAIN_URL).get('/video/av' + aid.to_s + '/')
        response.body.each_line { |line| 
            t = /<script type='text\/javascript'>EmbedPlayer\('player', "http:\/\/static.hdslb.com\/play.swf", "cid=(\d+)(?:.*?)"/.match(line);
            if t != nil
                return t[1]
            end
        }
    end
    
    # 视频弹幕(aid)
    def Video.commentWithAID(aid)
        Video.commentWithCID(Video.avToCID(aid))
    end
    
    # 视频弹幕(cid)
    def Video.commentWithCID(cid)
        uri = URI(K_BILIBILI_COMMENT_HOST + cid.to_s + '.xml')
        req = Net::HTTP::Get.new(uri)
        req['Accept-Encoding'] = 'deflate'
        res = Net::HTTP.start(uri.hostname, uri.port) {|http|
          http.request(req)
        }
        comments = Array.new
        Zlib::Inflate.new(-Zlib::MAX_WBITS).inflate(res.body).split("\n").each do |line|
            t = /<d p="(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?)">(.*)<\/d>/.match(line)
            if t != nil
                comment = Hash.new
                comment['time'] = t[1]
                comment['mode'] = t[2]
                comment['fontsize'] = t[3]
                comment['fontcolor'] = t[4]
                comment['timestamp'] = t[5]
                comment['pool'] = t[6]
                comment['sender'] = t[7]
                comment['rowID'] = t[8]
                comment['content'] = t[9]
                comments.push(comment)
            end
        end
        comments
    end
    
    # 获取下载链接(aid)
    def Video.downloadURLWithAID(aid, quality = 5)
        Video.downloadURLWithCID(Video.avToCID(aid), quality)
    end
    
    # 获取下载链接(cid)
    def Video.downloadURLWithCID(cid, quality = 5)
        response = Net::HTTP.new(K_BILIBILI_API_INTERFACE_URL).get(K_BILIBILI_API_VIDEO_DOWNLOAD + quality.to_s + '&cid=' + cid.to_s)
        JSON.parse(response.body)
    end
    
    # 视频弹幕
    def comment
        Video.commentWithCID(@cid)
    end
    
    # 视频分享量
    def shareCount
        response = Net::HTTP.new(K_BILIBILI_API_URL).get(K_BILIBILI_API_VIDEO_SHARE + @aid.to_s)
        result = JSON.parse(response.body)
        if result['code'] == 0
            return result['data']
        end
    end
    
    # 刷新视频信息
    def refresh
        return Video.av(@aid)
    end
    
    # 获取下载链接
    def downloadURL(quality = 5)
        Video.downloadURLWithCID(@cid, quality)
    end
end

class Bilibili
    def Bilibili.hi
        puts '哔哩哔哩(‾▽‾)～■□～(‾▽‾)Ruby'
    end

    # 通过uid获取用户
    def Bilibili.member(mid)
        if mid != nil
            response = Net::HTTP.new(K_BILIBILI_API_SPACE_URL).get(K_BILIBILI_API_SPACE_MEMBER_QUERY + mid.to_s)
            Member.member_with_JSON(JSON.parse(response.body))
        end
    end
    
    # 获取最近一周的番剧信息
    def Bilibili.bangumi(day = nil)
        if day != nil
            day = day.to_i % 7
        end
        response = Net::HTTP.new(K_BILIBILI_API_MAIN_URL).get(K_BILIBILI_API_INDEX_BANGUMI)
        bangumi = JSON.parse(response.body)['bangumi']['list']
        if day == nil
            return bangumi
        end
        result = Array.new
        bangumi.each do |filter|
            if filter['weekday'] == day
                result.push(filter)
            end
        end
        return result
    end
    
    # 通过av号获取视频信息
    def Bilibili.av(aid = nil)
        if aid != nil
            Video.av(aid)
        end
    end
end
